<?php

include_once 'template-menus.php';

/**
 * Return a themed breadcrumb trail.
 *
 * @param $breadcrumb
 *   An array containing the breadcrumb links.
 * @return
 *   A string containing the breadcrumb output.
 */
function canvas_breadcrumb($breadcrumb) {
  $show_breadcrumb = theme_get_setting('canvas_breadcrumb');
  $show_breadcrumb_home = theme_get_setting('canvas_breadcrumb_home');
  $breadcrumb_separator = theme_get_setting('canvas_breadcrumb_separator');
  $leading_separator = theme_get_setting('canvas_breadcrumb_leading') ? $breadcrumb_separator : '';
  $trailing_separator = (theme_get_setting('canvas_breadcrumb_trailing') || theme_get_setting('canvas_breadcrumb_title')) ? $breadcrumb_separator : '';

  // Determine if we are to display the breadcrumb
  if ($show_breadcrumb == 'yes' || $show_breadcrumb == 'admin' && arg(0) == 'admin') {
    if (!$show_breadcrumb_home) {
      // Optionally get rid of the homepage link
      array_shift($breadcrumb);
    }
    if (!empty($breadcrumb)) {
      // Return the breadcrumb with separators
      return '<div class="breadcrumb">' . $leading_separator . implode($breadcrumb_separator, $breadcrumb) . "$trailing_separator</div>";
    }
  }
  // Otherwise, return an empty string
  return '';
}


function canvas_preprocess_page(&$vars, $hook) {
  global $theme;

  // Optionally add the block editing styles.
  if (theme_get_setting('canvas_block_editing')) {
    drupal_add_css(canvas_theme_path() . '/block-editing.css', 'theme', 'all');
  }

  // Optionally add the wireframes style.
  if (theme_get_setting('canvas_wireframes')) {
    drupal_add_css(canvas_theme_path() . '/wireframes.css', 'theme', 'all');
  }

  $vars['css'] = drupal_add_css();
  $vars['styles'] = drupal_get_css();

  // Allow sub-themes to have an ie.css file
  $vars['canvas_theme_directory'] = canvas_theme_path();
  $vars['canvas_subtheme_directory'] = canvas_subtheme_path();

  // Add an optional title to the end of the breadcrumb.
  if (theme_get_setting('canvas_breadcrumb_title') && $vars['breadcrumb']) {
    $vars['breadcrumb'] = substr($vars['breadcrumb'], 0, -6) . $vars['title'] . '</div>';
  }

  // Classes for body element. Allows advanced theming based on context
  // (home page, node of certain type, etc.)
  $body_classes = array($vars['body_classes']);
  if (!$vars['is_front']) {
    // Add unique classes for each page and website section
    $path = drupal_get_path_alias($_GET['q']);
    list($section, ) = explode('/', $path, 2);
    $body_classes[] = canvas_id_safe('page-' . $path);
    $body_classes[] = canvas_id_safe('section-' . $section);
    if (arg(0) == 'node') {
      if (arg(1) == 'add') {
        if ($section == 'node') {
          array_pop($body_classes); // Remove 'section-node'
        }
        $body_classes[] = 'section-node-add'; // Add 'section-node-add'
      }
      elseif (is_numeric(arg(1)) && (arg(2) == 'edit' || arg(2) == 'delete')) {
        if ($section == 'node') {
          array_pop($body_classes); // Remove 'section-node'
        }
        $body_classes[] = 'section-node-' . arg(2); // Add 'section-node-edit' or 'section-node-delete'
      }
    }
  }

  // Add classes for primary, secondary links
  $map = 0;
  $link_labels = array(0 => 'no-links', 1 => 'primary-links', 2 => 'secondary-links', 3 => 'both-links');
  if ($vars['primary_links']) {
    $map |= 1;
  }
  if ($vars['secondary_links']) {
    $map |= 2;
  }
  $body_classes[] = $link_labels[$map];

  $vars['body_classes'] = implode(' ', $body_classes); // Concatenate with spaces

  canvas_page_features($vars, $hook);
}

/**
 * Special theme features
 * 
 * This function effectively overrides page.tpl.php because it manipulates $vars 
 * 
 */
function canvas_page_features(&$vars, $hook) {
  global $theme;
  $settings = theme_get_settings($theme);

  // Unconditionally theme primary/secondary links
  $vars['primary_links'] = theme('links', $vars['primary_links']);
  $vars['secondary_links'] = theme('links', $vars['secondary_links']);

  // Add linkback text
  if (isset($settings['canvas_linkback'])) {
    $vars['linkback'] = str_replace('[canvas_subtheme_directory]', url($vars['canvas_subtheme_directory']), $settings['canvas_linkback']);
  }

  // Add (optional) banner
  if (isset($settings['canvas_banner'])) {
    $vars['banner'] = $settings['canvas_banner'];
  }

  // Move some page elements into regions if specified by theme settings.
  // This is a fairly crude way to parametrically position things like
  //   breadcrumb, search_box into regions (like blocks).
  // It doesn't do any checking and blindly does what the .info file specifies.
  // Since the .info file is protected, this should be okay. 
  $layouts = $settings['canvas_layout'];
  if (is_array($layouts)) {
    foreach ($layouts as $item => $layout) {
      // Ignore non-arrays, Ignore circular reference
      if (!is_array($layout) || !isset($layout['region']) || $item == $layout['region'])
        continue;

      if (isset($vars[$item])) {
        $id = isset($layout['id']) ? (' id="' . $layout['id']. '"') : '';
        $class = isset($layout['class']) ? (' class="' . $layout['class']. '"') : '';
        $new_item = '<div' . $id . $class . '>' . $vars[$item] . '</div>';
        if (!empty($layout['append'])) {
          $vars[$layout['region']] .= $new_item;
        }
        else {
          $vars[$layout['region']] = $new_item . $vars[$layout['region']];
        }
        unset($vars[$item]);
      }
    }
  }

  // Insert extra div's for regions if specified (often needed for rounded corners) 
  $extradivs = $settings['canvas_extradivs'];
  if (is_array($extradivs)) {
    foreach ($extradivs as $region => $count) {
      $label = str_replace('_', '-', $region);
      if (isset($vars[$region]) && is_numeric($count)) {
        for ($i = $count; $i > 0; $i--) {
          $class = 'region-inner-' . $i . ' ';
          $class .= $label . '-inner-' . $i;
          $vars[$region] = '<div class="' . $class . '">' . $vars[$region] . '</div>';
        }
      }
    }
  }

  // Use pngfix.module if it exists, and if this theme says so
  // This code is inspired from pngfix_init()
  $pngfixclasses = $settings['canvas_pngfix'];
  if (!empty($pngfixclasses) && module_exists('pngfix')) {
    $jquerypngfixjs = drupal_get_path('module', 'pngfix') .'/jquery.pngFix.js';
    drupal_add_js($jquerypngfixjs, 'module');
    $js = "$(document).ready(function(){ $('$pngfixclasses').pngFix(); });";
    drupal_add_js($js, 'inline');

    // Re-fetch javascript scripts
    $vars['scripts'] = drupal_get_js();
  }
}


/**
 * Override or insert PHPTemplate variables into the node templates.
 *
 * @param $vars
 *   A sequential array of variables to pass to the theme template.
 * @param $hook
 *   The name of the theme function being called ("node" in this case.)
 */
function canvas_preprocess_node(&$vars, $hook) {
  global $user;
  $view_type = $vars['teaser'] ? 'teaser' : 'full';
  $node_type = canvas_id_safe($vars['type']);

  canvas_node_features($vars, $hook);

  // Special classes for nodes
  $node_classes = array();
  if ($vars['sticky']) {
    $node_classes[] = 'sticky';
  }
  if (!$vars['node']->status) {
    $node_classes[] = 'node-unpublished';
    $vars['unpublished'] = TRUE;
  }
  else {
    $vars['unpublished'] = FALSE;
  }
  if ($vars['node']->uid && $vars['node']->uid == $user->uid) {
    // Node is authored by current user
    $node_classes[] = 'node-mine';
  }
  $node_classes[] = 'node-' . $view_type;
  
  // Class for node type: "node-type-page", "node-type-story", "node-type-my-custom-type", etc.
  $node_classes[] = 'node-type-' . $node_type;
  $node_classes[] = 'node-type-' . $node_type . '-' . $view_type;
  $vars['node_classes'] = implode(' ', $node_classes); // Concatenate with spaces

  $node = $vars['node'];

  // Add a node-type-teaser/full suggestion
  $vars['template_files'][] = 'node-'. $node->type . '-' . $view_type;

  $vars['body_content'] = $node->content['body']['#value'];

  // If there is an extracted teaser image, do some processing
  if (isset($node->teaser_pp_img)) {
    $vars['teaser_pp_img'] = $node->content['teaser_pp_img']['#value'];
  }  
}

function canvas_node_features(&$vars, $hook) {
  $node = $vars['node'];

  // Split $vars['taxonomy'] into channels and non-channels
  $vid = variable_get('pp_ch_channel_vocabulary', FALSE);
  if (module_exists('pp_ch') && $vid && isset($node->taxonomy)) {
    $vars['taxonomy_channel'] = array();
    $vars['taxonomy_sans_channel'] = array();
    foreach ($node->taxonomy as $term) {
      if ($term->vid == $vid) {
        $vars['taxonomy_channel']['taxonomy_term_' . $term->tid] = $vars['taxonomy']['taxonomy_term_' . $term->tid];
      }
      else {
        $vars['taxonomy_sans_channel']['taxonomy_term_' . $term->tid] = $vars['taxonomy']['taxonomy_term_' . $term->tid];
      }
    }
    $vars['taxonomy_channel_terms'] = theme('links', $vars['taxonomy_channel'], array('class' => 'links inline'));
    $vars['taxonomy_sans_channel_terms'] = theme('links', $vars['taxonomy_sans_channel'], array('class' => 'links inline'));
  }
}

/**
 * CCK content exclusion
 *
 * This function tells CCK which fields NOT to include when rendering $node->content
 * 
 */
function canvas_content_exclude($content, $object, $context) {
  if ($object['type_name'] == 'story') {
    // Exclude these fields from $content so there is no duplication on $content when we render them separately
    if (in_array($object['field_name'], array('field_source', 'field_date', 'field_extended_status', 'field_subtitle'))) {
      return true;
    }
  }
}


/**
 * Override or insert PHPTemplate variables into the comment templates.
 *
 * @param $vars
 *   A sequential array of variables to pass to the theme template.
 * @param $hook
 *   The name of the theme function being called ("comment" in this case.)
 */
function canvas_preprocess_comment(&$vars, $hook) {
  global $user;

  // We load the node object that the current comment is attached to
  $node = node_load($vars['comment']->nid);
  // If the author of this comment is equal to the author of the node, we
  // set a variable so we can theme this comment uniquely.
  $vars['author_comment'] = $vars['comment']->uid == $node->uid ? TRUE : FALSE;

  $comment_classes = array();

  // Odd/even handling
  static $comment_odd = TRUE;
  $comment_classes[] = $comment_odd ? 'odd' : 'even';
  $comment_odd = !$comment_odd;

  if ($vars['comment']->status == COMMENT_NOT_PUBLISHED) {
    $comment_classes[] = 'comment-unpublished';
    $vars['unpublished'] = TRUE;
  }
  else {
    $vars['unpublished'] = FALSE;
  }
  if ($vars['author_comment']) {
    // Comment is by the node author
    $comment_classes[] = 'comment-by-author';
  }
  if ($vars['comment']->uid == 0) {
    // Comment is by an anonymous user
    $comment_classes[] = 'comment-by-anon';
  }
  if ($user->uid && $vars['comment']->uid == $user->uid) {
    // Comment was posted by current user
    $comment_classes[] = 'comment-mine';
  }
  $vars['comment_classes'] = implode(' ', $comment_classes);

  // If comment subjects are disabled, don't display 'em
  if (variable_get('comment_subject_field', 1) == 0) {
    $vars['title'] = '';
  }
}

function canvas_preprocess_content_field(&$variables) {  
  // Some customisations for story content type
  // Turn off some field labels for story
  if ($variables['node']->type == 'story' && in_array($variables['field_name'], array('field_subtitle', 'field_source', 'field_date'))) {
    $variables['label_display'] = 'hidden';
  }

  // CCK field items default to using <div>
  $variables['field_list_start_tag'] = '';
  $variables['field_list_end_tag'] = '';
  $variables['field_list_item_tag'] = 'div';

  // For some cases, change this to <ul> and <li>
  if (($variables['field_name'] == 'field_headlines' && $variables['teaser']) ||
      ($variables['field_name'] == 'field_stories')) {
    $variables['field_list_start_tag'] = '<ul>';
    $variables['field_list_end_tag'] = '</ul>';
    $variables['field_list_item_tag'] = 'li';
  }
}

/**
 * Override or insert PHPTemplate variables into the block templates.
 *
 * @param $vars
 *   A sequential array of variables to pass to the theme template.
 * @param $hook
 *   The name of the theme function being called ("block" in this case.)
 */
function canvas_preprocess_block(&$vars, $hook) {
  $block = $vars['block'];

  // Special classes for blocks
  $block_classes = array();
  $block_classes[] = 'block-' . $block->module;
  $block_classes[] = 'region-' . $vars['block_zebra'];
  $block_classes[] = $vars['zebra'];
  $block_classes[] = 'region-count-' . $vars['block_id'];
  $block_classes[] = 'count-' . $vars['id'];
  $vars['block_classes'] = implode(' ', $block_classes);

  $vars['edit_links'] = '';
  if (theme_get_setting('canvas_block_editing') && user_access('administer blocks')) {
    // Display 'edit block' for custom blocks
    if ($block->module == 'block') {
      $edit_links[] = l('<span>' . t('edit block') . '</span>', 'admin/build/block/configure/' . $block->module . '/' . $block->delta,
        array(
          'attributes' => array(
            'title' => t('edit the content of this block'),
            'class' => 'block-edit',
          ),
          'query' => drupal_get_destination(),
          'html' => TRUE,
        )
      );
    }
    // Display 'configure' for other blocks
    else {
      $edit_links[] = l('<span>' . t('configure') . '</span>', 'admin/build/block/configure/' . $block->module . '/' . $block->delta,
        array(
          'attributes' => array(
            'title' => t('configure this block'),
            'class' => 'block-config',
          ),
          'query' => drupal_get_destination(),
          'html' => TRUE,
        )
      );
    }

    // Display 'administer views' for views blocks
    if ($block->module == 'views' && user_access('administer views')) {
      $edit_links[] = l('<span>' . t('edit view') . '</span>', 'admin/build/views/' . $block->delta . '/edit',
        array(
          'attributes' => array(
            'title' => t('edit the view that defines this block'),
            'class' => 'block-edit-view',
          ),
          'query' => drupal_get_destination(),
          'fragment' => 'edit-block',
          'html' => TRUE,
        )
      );
    }
    // Display 'edit menu' for menu blocks
    elseif (($block->module == 'menu' || ($block->module == 'user' && $block->delta == 1)) && user_access('administer menu')) {
      $menu_name = ($block->module == 'user') ? 'navigation' : $block->delta;
      $edit_links[] = l('<span>' . t('edit menu') . '</span>', 'admin/build/menu-customize/' . $menu_name,
        array(
          'attributes' => array(
            'title' => t('edit the menu that defines this block'),
            'class' => 'block-edit-menu',
          ),
          'query' => drupal_get_destination(),
          'html' => TRUE,
        )
      );
    }
    $vars['edit_links_array'] = $edit_links;
    $vars['edit_links'] = '<div class="edit">' . implode(' ', $edit_links) . '</div>';
  }
}

function canvas_id_safe($string) {
  // Replace with dashes anything that isn't A-Z, numbers, dashes, or underscores.
  $string = strtolower(preg_replace('/[^a-zA-Z0-9-]+/', '-', $string));
  // If the first character is not a-z, add 'id' in front.
  if (!ctype_lower($string{0})) { // Don't use ctype_alpha since its locale aware.
    $string = 'id' . $string;
  }
  return $string;
}

/* path_to_theme() is inconsistent and non-intuitive. Use these instead.  */
function canvas_theme_path() {
  static $path;
  if (!isset($path)) {
    $path = drupal_get_path('theme', 'canvas');
  }
  return $path;
}

function canvas_subtheme_path() {
  global $theme;
  static $path;
  if (!isset($path)) {
    $path = drupal_get_path('theme', $theme);
  }
  return $path;
}

function canvas_theme(&$existing, $type, $theme, $path) {
  // Each theme has two possible preprocess functions that can act on a hook.
  // This function applies to every hook.
  $functions[0] = $theme . '_preprocess';
  // Inspect the preprocess functions for every hook in the theme registry.
  foreach ($existing AS $hook => $value) {
    // Each theme has two possible preprocess functions that can act on a hook.
    // This function only applies to this hook.
    $functions[1] = $theme . '_preprocess_' . $hook;
    foreach ($functions AS $key => $function) {
      // Add any functions that are not already in the registry.
      if (function_exists($function) && !in_array($function, $existing[$hook]['preprocess functions'])) {
        // We add the preprocess function to the end of the existing list.
        $existing[$hook]['preprocess functions'][] = $function;
      }
    }
  }

  // Since we modify the $existing cache directly, return nothing.
  return array();
}

/**
 * Read the theme settings' default values from the .info and save them into the database.
 *
 * @param $theme
 *   The actual name of theme that is being being checked.
 */

function canvas_settings_init($theme) {
  $themes = list_themes();

  // Get the default values and settings api version from canvas.info
  $defaults = $themes['canvas']->info['settings'];
  $settings_api_version = $defaults['canvas_settings_api_version'];

  // If there is an existing setting, then don't reset theme settings
  if (!is_null(theme_get_setting('canvas_settings_api_version')) && theme_get_setting('canvas_settings_api_version') >= $settings_api_version) {
    return;
  }
  
  // Get the default values from the subtheme's .info file.
  $defaults = array_merge($defaults, $themes[$theme]->info['settings']);

  // Get the theme settings saved in the database.
  $settings = theme_get_settings($theme);
  // Don't save the toggle_node_info_ variables.
  if (module_exists('node')) {
    foreach (node_get_types() as $type => $name) {
      unset($settings['toggle_node_info_' . $type]);
    }
  }

  // Overwrite settings_api_version 
  $settings['canvas_settings_api_version'] = $settings_api_version;
  
  // Save default theme settings.
  variable_set(
    str_replace('/', '_', 'theme_' . $theme . '_settings'),
    array_merge($defaults, $settings)
  );
  // Force refresh of Drupal internals.
  theme_get_setting('', TRUE);
}

if (!defined('MAINTENANCE_MODE')) {
  global $theme_key;
  canvas_settings_init($theme_key);
}

function canvas_tinymce_content_css() 
{
  global $theme;
  if (file_exists(canvas_subtheme_path() . '/tinymce_content.css')) {
    return canvas_subtheme_path() . '/tinymce_content.css';
  }
  else if (file_exists(canvas_theme_path() . '/tinymce_content.css')) {
    return canvas_theme_path() . '/tinymce_content.css';
  }
  else {
    return theme_tinymce_content_css();
  }
}
